@@ -1,11 +1,8 @@
-/*  $Id: article.c,v 1.33 2003/01/13 07:59:32 rra Exp $
+/*  $Id: article.c 7285 2005-06-07 06:38:24Z eagle $
 **
 **  The Article class for innfeed.
 **
 **  Written by James Brister <brister@vix.com>
-**  Copyright 1996 by the Internet Software Consortium
-**
-**  For license terms, see the end of this file.
 **
 **  The implementation of the Article class. Articles are the abstraction for
 **  the actual news articles. They are a reference counted object because they
@@ -47,6 +44,16 @@
 
 extern bool useMMap ;
 
+struct map_info_s {
+    ARTHANDLE *arthandle;
+    const void *mMapping;
+    size_t size;
+    int refCount;
+    struct map_info_s *next;
+};
+
+typedef struct map_info_s *MapInfo;
+
 struct article_s 
 {
     int refCount ;              /* the reference count on this article */
@@ -54,11 +61,10 @@ struct article_s
     char *msgid ;               /* the msgid of the article (INN tells us) */
     Buffer contents ;           /* the buffer of the actual on disk stuff */
     Buffer *nntpBuffers ;       /* list of buffers for transmisson */
-    const void *mMapping ;      /* base of memory mapping, or NULL if none */
+    MapInfo mapInfo;		/* arthandle and mMapping */
     bool loggedMissing ;        /* true if article is missing and we logged */
     bool articleOk ;            /* true until we know otherwise. */
     bool inWireFormat ;         /* true if ->contents is \r\n/dot-escaped */
-    ARTHANDLE *arthandle ;      /* Storage manager article handle */
 } ;
 
 struct hash_entry_s {
@@ -96,7 +102,7 @@ static bool artFreeContents (Article art) ;  /* Tell the Article to release
                                                 its contents buffer if
                                                 possible. */
 
-static void artUnmap (Article art, size_t size) ; /* munmap an mmap()ed
+static void artUnmap (Article art) ; /* munmap an mmap()ed
                                                      article */
 
 
@@ -159,7 +165,7 @@ static unsigned int articleTotal ; /* number of articles alloced since last log.
 
 static TimeoutId articleStatsId ; /* The timer callback id. */
 
-
+static MapInfo mapInfo;
 
   /*
    * Hash Table data
@@ -212,12 +218,11 @@ Article newArticle (const char *filename, const char *msgid)
       newArt->msgid = xstrdup (msgid) ;
       
       newArt->contents = NULL ;
-      newArt->mMapping = NULL ;
+      newArt->mapInfo = NULL ;
       newArt->refCount = 1 ;
       newArt->loggedMissing = false ;
       newArt->articleOk = true ;
       newArt->inWireFormat = false ;
-      newArt->arthandle = NULL;
       
       d_printf (3,"Adding a new article(%p): %s\n", (void *)newArt, msgid) ;
       
@@ -260,8 +265,8 @@ void delArticle (Article article)
 
       if (article->contents != NULL)
         {
-          if (article->mMapping)
-            artUnmap(article, bufferSize(article->contents)) ;
+          if (article->mapInfo)
+            artUnmap(article);
           else
             bytesInUse -= bufferDataSize (article->contents) ;
       
@@ -467,17 +472,67 @@ static Buffer artGetContents (Article article)
   return rval ;
 }
 
+  /* arthandle/mMapping needs to be refcounted since a buffer
+     may exist referencing it after delArticle if the remote
+     host sends the return status too soon (diablo, 439). */
 
-static void artUnmap (Article article, size_t size) {
-    
-    if (article->arthandle)
-	SMfreearticle(article->arthandle);
+static MapInfo getMapInfo(ARTHANDLE *arthandle,
+		const void *mMapping, size_t size)
+{
+  MapInfo m;
+
+  for (m = mapInfo; m; m = m->next) {
+    if (m->arthandle == arthandle &&
+        m->mMapping == mMapping) {
+      m->refCount++;
+      return m;
+    }
+  }
+
+  m = xmalloc(sizeof(struct map_info_s));
+  m->refCount = 1;
+  m->arthandle = arthandle;
+  m->mMapping = mMapping;
+  m->size = size;
+  m->next = mapInfo;
+  mapInfo = m;
+
+  return m;
+}
+
+static void delMapInfo(void *vm)
+{
+    MapInfo i, prev;
+    MapInfo m = (MapInfo)vm;
+
+    if (m == NULL)
+      return;
+
+    if (--(m->refCount) > 0)
+      return;
+
+    if (m->arthandle)
+      SMfreearticle(m->arthandle);
     else
-	if (munmap(article->mMapping, size) < 0)
-	    syslog (LOG_NOTICE, "munmap %s: %m", article->fname) ;
+      if (munmap(m->mMapping, m->size) < 0)
+	syslog (LOG_NOTICE, "munmap article: %m");
 
-    article->arthandle = NULL;
-    article->mMapping = NULL ;
+    prev = NULL;
+    for (i = mapInfo; i != m; i = i->next)
+      prev = i;
+
+    if (prev)
+      prev->next = m->next;
+    else
+      mapInfo = m->next;
+
+    free(m);
+}
+
+static void artUnmap (Article article) {
+
+    delMapInfo(article->mapInfo);
+    article->mapInfo = NULL;
 }
 
 
@@ -516,7 +571,9 @@ static bool fillContents (Article article)
     size_t idx = 0, amtToRead ;
     size_t newBufferSize ;
     HashEntry h ;
-  
+    ARTHANDLE *arthandle = NULL;
+    const void *mMapping = NULL;
+
     ASSERT (article->contents == NULL) ;
     
     TMRstart(TMR_READART);
@@ -527,9 +584,9 @@ static bool fillContents (Article article)
 	avgCharsPerLine = 75 ;      /* roughly number of characters per line */
     
     if (IsToken(article->fname)) {
-	opened = ((article->arthandle = SMretrieve(TextToToken(article->fname), RETR_ALL)) != NULL) ? true : false;
+	opened = ((arthandle = SMretrieve(TextToToken(article->fname), RETR_ALL)) != NULL) ? true : false;
 	if (opened)
-	    articlesize = article->arthandle->len;
+	    articlesize = arthandle->len;
 	else {
 	    if (SMerrno != SMERR_NOENT && SMerrno != SMERR_UNINIT) {
 		syslog(LOG_ERR, "Could not retrieve %s: %s",
@@ -543,7 +600,7 @@ static bool fillContents (Article article)
 	struct stat sb ;
 	
 	opened = ((fd = open (article->fname,O_RDONLY,0)) >= 0) ? true : false;
-	article->arthandle = NULL;
+	arthandle = NULL;
 	if (opened) {
 	    if (fstat (fd, &sb) < 0) {
 		article->articleOk = false ;
@@ -583,23 +640,27 @@ static bool fillContents (Article article)
     amtToRead = articlesize ;
     newBufferSize = articlesize ;
     
-    if (article->arthandle || useMMap) {
-	if (article->arthandle)
-	    article->mMapping = article->arthandle->data;
+    if (arthandle || useMMap) {
+	if (arthandle)
+	    mMapping = arthandle->data;
 	else 
-	    article->mMapping = mmap(NULL, articlesize, PROT_READ,
+	    mMapping = mmap(NULL, articlesize, PROT_READ,
                                      MAP_SHARED, fd, 0);
 	
-	if (article->mMapping == MAP_FAILED) {
+	if (mMapping == MAP_FAILED) {
 	    /* dunno, but revert to plain reading */
-	    article->mMapping = NULL ;
+	    mMapping = NULL ;
             syswarn ("ME mmap failure %s (%s)",
                      article->fname,
                      strerror(errno)) ;
 	} else {
-	    article->contents = newBufferByCharP(article->mMapping,
+	    article->contents = newBufferByCharP(mMapping,
 						 articlesize,
 						 articlesize);
+	    article->mapInfo = getMapInfo(arthandle, mMapping, articlesize);
+	    article->mapInfo->refCount++; /* one more for the buffer */
+	    bufferSetDeletedCbk(article->contents, delMapInfo, article->mapInfo);
+
 	    buffer = bufferBase (article->contents) ;
 	    if ((p = strchr(buffer, '\n')) == NULL) {
 		article->articleOk = false;
@@ -639,7 +700,8 @@ static bool fillContents (Article article)
 	if ((amtToRead + bytesInUse) > maxBytesInUse && maxLimitNotified == false) {
 	    maxLimitNotified = true ;
             notice ("ME exceeding maximum article byte limit: %d (max),"
-                    " %d (cur)", maxBytesInUse, amtToRead + bytesInUse) ;
+                    " %lu (cur)", maxBytesInUse, 
+                    (unsigned long) (amtToRead + bytesInUse)) ;
 	}
 	
 	if ((article->contents = newBuffer (newBufferSize)) == NULL)
@@ -650,9 +712,9 @@ static bool fillContents (Article article)
 	    byteTotal += articlesize ;
 	}
 	
-	if (article->mMapping && buffer != NULL) {               
-	    memcpy(buffer, article->mMapping, articlesize);
-	    artUnmap(article, articlesize) ;
+	if (mMapping && buffer != NULL) {               
+	    memcpy(buffer, mMapping, articlesize);
+	    artUnmap(article) ;
 	    amtToRead = 0;
 	}
 	
@@ -723,7 +785,7 @@ static bool fillContents (Article article)
 
     
     /* If we're not useing storage api, we should close a valid file descriptor */
-    if (!article->arthandle && (fd >= 0))
+    if (!arthandle && (fd >= 0))
 	close (fd) ;
 
     TMRstop(TMR_READART);
@@ -816,6 +878,10 @@ static bool prepareArticleForNNTP (Article article)
       nntpBuffs [0] = newBufferByCharP (bufferBase (contents),
                                         bufferDataSize (contents),
                                         bufferDataSize (contents)) ;
+      if (article->mapInfo) {
+	article->mapInfo->refCount++;
+	bufferSetDeletedCbk(nntpBuffs[0], delMapInfo, article->mapInfo);
+      }
       nntpBuffs [1] = NULL ;
     }
   
@@ -847,8 +913,8 @@ static bool artFreeContents (Article art)
 
   ASSERT (bufferRefCount (art->contents) == 1) ;
 
-  if (art->mMapping)
-    artUnmap(art, bufferSize(art->contents)) ;
+  if (art->mapInfo)
+    artUnmap(art);
   else
     bytesInUse -= bufferDataSize (art->contents) ;
   
@@ -993,20 +1059,3 @@ static void hashValidateTable (void)
     }
 #endif
 }
-
-/*
-**  Copyright 1996 by the Internet Software Consortium
-**
-**  Permission to use, copy, modify, and distribute this software for any
-**  purpose with or without fee is hereby granted, provided that the above
-**  copyright notice and this permission notice appear in all copies.
-**
-**  THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
-**  DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
-**  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
-**  INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
-**  OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
-**  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
-**  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-**  PERFORMANCE OF THIS SOFTWARE.
-*/
