@@ -1,4 +1,4 @@
-/*  $Id: misc.c,v 1.29.2.1 2004/01/25 20:07:10 rra Exp $
+/*  $Id: misc.c 7420 2005-10-09 04:40:13Z eagle $
 **
 **  Helper routines for the innfeed program.
 **
@@ -154,7 +154,7 @@ void logOrPrint (int level, FILE *fp, const char *fmt, ...)
       char buffer [512] ;      /* gag me */
 
       vsnprintf (buffer,sizeof (buffer),fmt,ap) ;
-      syslog (level,buffer) ;
+      syslog (level,"%s",buffer) ;
     }
   va_end (ap) ;
 }
