@@ -1,4 +1,4 @@
-/*  $Id: startinnfeed.c,v 1.23 2003/01/19 01:15:32 rra Exp $
+/*  $Id: startinnfeed.c 7065 2004-12-19 21:49:18Z rra $
 **
 **  Raise system limits and exec innfeed.
 **
@@ -91,9 +91,6 @@ main(int argc, char *argv[])
 # ifdef RLIMIT_DATA
     setrlimit(RLIMIT_DATA, &rl);
 # endif
-# ifdef RLIMIT_STACK
-    setrlimit(RLIMIT_STACK, &rl);
-# endif
 #endif /* HAVE_SETRLIMIT */
 
     /* Permanently drop privileges. */
