@@ -1,16 +1,16 @@
-/*  $Revision: 1.4 $
+/*  $Id: sendpass.c 7145 2005-04-10 03:28:01Z rra $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
+
+#include "inn/innconf.h"
+#include "libinn.h"
 #include "nntp.h"
 #include "paths.h"
-#include "libinn.h"
-#include "macros.h"
 
 
 /*
@@ -20,6 +20,7 @@ int NNTPsendpassword(char *server, FILE *FromServer, FILE *ToServer)
 {
     FILE	        *F;
     char	        *p;
+    char                *path;
     char		buff[SMBUF];
     char		input[SMBUF];
     char		*user;
@@ -27,21 +28,27 @@ int NNTPsendpassword(char *server, FILE *FromServer, FILE *ToServer)
     char		*style;
     int			oerrno;
 
-    /* What server are we interested in?  Default to the campus one. */
-    if (server == NULL
-     && (server = innconf->server) == NULL)
-	return -1;
+    /* Default to innconf->server.  If that's not set either, error out.  Fake
+       errno since some of our callers rely on it. */
+    if (server == NULL)
+        server = innconf->server;
+    if (server == NULL) {
+        errno = EINVAL;
+        return -1;
+    }
 
     /* Open the password file; coarse check on errno, but good enough. */
-    if ((F = fopen(cpcatpath(innconf->pathetc, _PATH_NNTPPASS), "r")) == NULL) {
+    path = concatpath(innconf->pathetc, _PATH_NNTPPASS);
+    F = fopen(path, "r");
+    free(path);
+    if (F == NULL)
 	return errno == EPERM ? -1 : 0;
-    }
 
     /* Scan the file, skipping blank and comment lines. */
     while (fgets(buff, sizeof buff, F) != NULL) {
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
-	if (buff[0] == '\0' || buff[0] == COMMENT_CHAR)
+	if (buff[0] == '\0' || buff[0] == '#')
 	    continue;
 
 	/* Parse the line. */
@@ -53,18 +60,18 @@ int NNTPsendpassword(char *server, FILE *FromServer, FILE *ToServer)
 	*pass++ = '\0';
 	if ((style = strchr(pass, ':')) != NULL) {
 	    *style++ = '\0';
-	    if (!EQ(style, "authinfo")) {
+	    if (strcmp(style, "authinfo") != 0) {
 		errno = EDOM;
 		break;
 	    }
 	}
 
-	if (!caseEQ(server, buff))
+	if (strcasecmp(server, buff) != 0)
 	    continue;
 
 	if (*user) {
 	    /* Send the first part of the command, get a reply. */
-	    (void)fprintf(ToServer, "authinfo user %s\r\n", user);
+	    fprintf(ToServer, "authinfo user %s\r\n", user);
 	    if (fflush(ToServer) == EOF || ferror(ToServer))
 		break;
 	    if (fgets(input, sizeof input, FromServer) == NULL
@@ -74,7 +81,7 @@ int NNTPsendpassword(char *server, FILE *FromServer, FILE *ToServer)
 
 	if (*pass) {
 	    /* Send the second part of the command, get a reply. */
-	    (void)fprintf(ToServer, "authinfo pass %s\r\n", pass);
+	    fprintf(ToServer, "authinfo pass %s\r\n", pass);
 	    if (fflush(ToServer) == EOF || ferror(ToServer))
 		break;
 	    if (fgets(input, sizeof input, FromServer) == NULL
@@ -83,19 +90,19 @@ int NNTPsendpassword(char *server, FILE *FromServer, FILE *ToServer)
 	}
 
 	/* Authenticated. */
-	(void)fclose(F);
+	fclose(F);
 	return 0;
     }
 
     /* End of file without finding a password, that's okay. */
     if (feof(F)) {
-	(void)fclose(F);
+	fclose(F);
 	return 0;
     }
 
     /* Save errno, close the file, fail. */
     oerrno = errno;
-    (void)fclose(F);
+    fclose(F);
     errno = oerrno;
     return -1;
 }
