@@ -1,4 +1,4 @@
-/*  $Id: article.c,v 1.143 2003/02/17 11:19:58 alexk Exp $
+/*  $Id: article.c 7267 2005-06-06 05:10:01Z eagle $
 **
 **  Article-related routines.
 */
@@ -489,14 +489,16 @@ char *GetHeader(const char *header)
     /* Bogus value here to make sure that it isn't initialized to \n */
     char		lastchar = ' ';
     const char		*limit;
+    const char		*cmplimit;
     static char		*retval = NULL;
     static int		retlen = 0;
     int			headerlen;
     bool		pathheader = false;
     bool		xrefheader = false;
 
-    limit = ARThandle->data + ARThandle->len - strlen(header) - 1;
-    for (p = ARThandle->data; p < limit; p++) {
+    limit = ARThandle->data + ARThandle->len;
+    cmplimit = ARThandle->data + ARThandle->len - strlen(header) - 1;
+    for (p = ARThandle->data; p < cmplimit; p++) {
 	if (*p == '\r')
 	    continue;
 	if ((lastchar == '\n') && (*p == '\n')) {
@@ -961,33 +963,6 @@ void CMDxover(int ac, char *av[])
 
 }
 
-static bool
-build_groups(char *buff)
-{
-    char *next, *p, *q, *newsgroupbuff;
-
-    newsgroupbuff = p = xstrdup(buff);
-    if ((p = strchr(p, ' ')) == NULL) {
-	free(newsgroupbuff);
-	return false;
-    }
-
-    for (buff[0] = '\0', q = buff, p++; *p != '\0'; ) {
-	if ((next = strchr(p, ':')) == NULL) {
-	    free(newsgroupbuff);
-	    return false;
-	}
-	*next++ = '\0';
-	strcat(q, p);
-	q += (next - p - 1);
-	if ((p = strchr(next, ' ')) == NULL)
-	    break;
-	*p = ',';
-    }
-    free(newsgroupbuff);
-    return true;
-}
-
 /*
 **  XHDR and XPAT extensions.  Note that HDR as specified in the new NNTP
 **  draft works differently than XHDR has historically, so don't just use this
@@ -1011,7 +986,6 @@ void CMDpat(int ac, char *av[])
     char                *data;
     int                 len;
     TOKEN               token;
-    bool                BuildingNewsgroups = false;
     struct cvector *vector = NULL;
 
     if (!PERMcanread) {
@@ -1062,11 +1036,7 @@ void CMDpat(int ac, char *av[])
 	}
 
 	/* In overview? */
-	if (strcasecmp(header, "Newsgroups") == 0) {
-	    BuildingNewsgroups = true;
-	    Overview = overview_index("Xref", OVextra);
-	} else
-	    Overview = overview_index(header, OVextra);
+        Overview = overview_index(header, OVextra);
 
 	/* Not in overview, we have to fish headers out from the articles */
 	if (Overview < 0 ) {
@@ -1106,12 +1076,7 @@ void CMDpat(int ac, char *av[])
 	    vector = overview_split(data, len, NULL, vector);
 	    p = overview_getheader(vector, Overview, OVextra);
 	    if (p != NULL) {
-		if (BuildingNewsgroups) {
-		    if (!build_groups(p)) {
-			free(p);
-			continue;
-		    }
-		} else if (PERMaccessconf->virtualhost &&
+		if (PERMaccessconf->virtualhost &&
 			   Overview == overhdr_xref) {
 		    p = vhost_xref(p);
 		    if (p == NULL)
