@@ -1,4 +1,4 @@
-/*  $Revision: 1.94.2.4 $
+/*  $Revision: 7450 $
 **
 **  Check article, send it to the local server.
 */
@@ -20,7 +20,6 @@ char	**OtherHeaders;
 int	OtherCount;
 bool   HeadersModified;
 static int	OtherSize;
-static bool	WasMailed;
 static const char * const BadDistribs[] = {
     BAD_DISTRIBS
 };
@@ -336,22 +335,17 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
 	}
     }
 
+    /* If authorized, add the header based on our info.  If not authorized,
+       zap the Sender so we don't put out unauthenticated data. */
     if (PERMaccessconf->nnrpdauthsender) {
-	/* If authorized, add the header based on our info.  If not
-         * authorized, zap the Sender so we don't put out unauthenticated
-         * data. */
-	if (PERMauthorized) {
-	    if (PERMuser[0] == '\0') {
-		snprintf(sendbuff, sizeof(sendbuff), "%s@%s", "UNKNOWN",
+	if (PERMauthorized && PERMuser[0] != '\0') {
+            p = strchr(PERMuser, '@');
+            if (p == NULL) {
+                snprintf(sendbuff, sizeof(sendbuff), "%s@%s", PERMuser,
                          ClientHost);
-	    } else {
-		if ((p = strchr(PERMuser, '@')) == NULL) {
-		    snprintf(sendbuff, sizeof(sendbuff), "%s@%s", PERMuser,
-                             ClientHost);
-		} else {
-		    snprintf(sendbuff, sizeof(sendbuff), "%s", PERMuser);
-		}
-	    }
+            } else {
+                snprintf(sendbuff, sizeof(sendbuff), "%s", PERMuser);
+            }
 	    HDR_SET(HDR__SENDER, sendbuff);
 	} else {
 	    HDR_SET(HDR__SENDER, NULL);
@@ -658,7 +652,6 @@ MailArticle(char *group, char *article)
                  i, "Article might not have been mailed");
 	return Error;
     }
-    WasMailed = true;
     return NULL;
 }
 
@@ -690,6 +683,7 @@ ValidNewsgroups(char *hdr, char **modgroup)
     groups = xstrdup(hdr);
     if ((p = strtok(groups, NGSEPS)) == NULL)
 	return "Can't parse newsgroups line";
+    Error[0] = '\0';
 
     /* Reject all articles with Approved headers unless the user is allowed to
        add them, even to unmoderated or local groups.  We want to reject them
@@ -701,7 +695,6 @@ ValidNewsgroups(char *hdr, char **modgroup)
                  "You are not allowed to approve postings");
     }
 
-    Error[0] = '\0';
     FoundOne = false;
     h = DDstart((FILE *)NULL, (FILE *)NULL);
     do {
@@ -984,7 +977,6 @@ ARTpost(char *article,
 
     /* Basic processing. */
     OtherCount = 0;
-    WasMailed = false;
     for (hp = Table; hp < ARRAY_END(Table); hp++) {
 	hp->Size = strlen(hp->Name);
 	hp->Value = hp->Body = NULL;
