@@ -1,20 +1,16 @@
 #
-# $Id: filter_innd.pl 9247 2011-07-17 18:03:49Z iulius $
+# $Id: filter_innd.pl 3048 2000-04-04 23:40:09Z rra $
 #
 # Sample Perl filtering file for the innd hooks.
 # 
 
-# This file gets loaded at innd process startup, and everytime a
-# "ctlinnd reload filter.perl 'reason'" or a
-# "ctlinnd reload all 'reason'" is done.
+# This file gets loaded at innd process startup, and everytime a 'reload
+# perl xxx' or a 'reload all xxx' is done.
 #
 # Before this file is loaded, the perl routine `filter_before_reload' is
 # called, and after it's finished loading, the perl routine
 # `filter_after_reload' is called. See startup_innd.pl for more details.
 #
-# When filtering is disabled, the filter_end() Perl routine is called,
-# if defined, prior to the deactivation of the filter.
-#
 # The following routines can be defined here for use by innd:
 #
 #	sub filter_art { ... }
@@ -25,9 +21,7 @@
 #		global associative array `%hdr.'  If it returns the empty
 #		string ("") then the article is accepted. If it returns any
 #		non-null string value, then the article is rejected and the
-#		returned string value is logged as the reason why (make sure
-#               that such a message is properly encoded in UTF-8 so as to comply
-#               with the NNTP protocol).
+#		returned string value is logged as the reason why.
 #
 #		The standard headers are:
 #
@@ -61,10 +55,8 @@
 #		This routine is called when each article (in streaming
 #		mode only) is checked to see if INN wants to accept the
 #		article.  If it returns the empty string, the article
-#		is accepted.  If it returns a non-empty value, the
-#		article is refused (make sure that such a message is
-#               properly encoded in UTF-8 so as to comply with the
-#               NNTP protocol).  It is called with one argument,
+#		is accepted. If it returns a non-empty value, the
+#		article is refused.  It is called with one argument,
 #		the message-id to check.
 
 
@@ -79,7 +71,7 @@ sub filter_art {
 	my $rval = "" ; # Assume we'll accept. Cannot be `0'
 
 ### Remove two leading '##' from the following section (and then
-### "ctlinnd reload filter.perl 'reason'" and the filter will reject articles that
+### 'ctlinnd reload filter.perl' and the filter will reject articles that
 ### have "make money" in the subject, or are posted to more than 10
 ### newsgroups. 
 
@@ -127,9 +119,7 @@ sub filter_messageid {
      $rval;
 }
 
-sub filter_end {
-# Do whatever you want to clean up things when Perl filtering is disabled.
-}
+
 
 
 
