@@ -1,4 +1,4 @@
-/*  $Id: tdx-group.c,v 1.22 2003/01/21 09:19:33 alexk Exp $
+/*  $Id: tdx-group.c 6593 2004-01-05 21:40:07Z rra $
 **
 **  Group index handling for the tradindexed overview method.
 **
@@ -95,6 +95,7 @@
 #include "inn/messages.h"
 #include "inn/mmap.h"
 #include "inn/qio.h"
+#include "inn/vector.h"
 #include "libinn.h"
 #include "paths.h"
 #include "tdx-private.h"
@@ -501,7 +502,7 @@ index_find(struct group_index *index, const char *group)
 	return -1;
     loc = index->header->hash[index_bucket(hash)].recno;
 
-    while (loc >= 0) {
+    while (loc >= 0 && loc < index->count) {
         struct group_entry *entry;
 
         if (loc > index->count && !index_maybe_remap(index, loc))
@@ -548,7 +549,7 @@ index_unlink_hash(struct group_index *index, HASH hash)
     parent = &index->header->hash[index_bucket(hash)].recno;
     current = *parent;
 
-    while (current >= 0) {
+    while (current >= 0 && current < index->count) {
         struct group_entry *entry;
 
         if (current > index->count && !index_maybe_remap(index, current))
@@ -972,6 +973,15 @@ tdx_expire(const char *group, ARTNUM *low, struct history *history)
 struct hashmap {
     HASH hash;
     char *name;
+    char flag;
+};
+
+/* Holds information needed by hash traversal functions.  Right now, this is
+   just the pointer to the group index and a flag saying whether to fix
+   problems or not. */
+struct audit_data {
+    struct group_index *index;
+    bool fix;
 };
 
 
@@ -1026,7 +1036,8 @@ hashmap_load(void)
 {
     struct hash *hash;
     QIOSTATE *active;
-    char *activepath, *line, *p;
+    char *activepath, *line;
+    struct cvector *data = NULL;
     struct stat st;
     size_t hash_size;
     struct hashmap *group;
@@ -1046,16 +1057,20 @@ hashmap_load(void)
 
     line = QIOread(active);
     while (line != NULL) {
-        p = strchr(line, ' ');
-        if (p != NULL)
-            *p = '\0';
+        data = cvector_split_space(line, data);
+        if (data->count != 4) {
+            warn("tradindexed: malformed active file line %s", line);
+            continue;
+        }
         group = xmalloc(sizeof(struct hashmap));
-        group->name = xstrdup(line);
+        group->name = xstrdup(data->strings[0]);
+        group->flag = data->strings[3][0];
         grouphash = Hash(group->name, strlen(group->name));
         memcpy(&group->hash, &grouphash, sizeof(HASH));
         hash_insert(hash, &group->hash, group);
         line = QIOread(active);
     }
+    cvector_free(data);
     QIOclose(active);
     return hash;
 }
@@ -1075,7 +1090,8 @@ tdx_index_print(const char *name, const struct group_entry *entry,
 {
     fprintf(output, "%s %lu %lu %lu %lu %c %lu %lu\n", name, entry->high,
             entry->low, entry->base, (unsigned long) entry->count,
-            entry->flag, entry->deleted, (unsigned long) entry->indexinode);
+            entry->flag, (unsigned long) entry->deleted,
+            (unsigned long) entry->indexinode);
 }
 
 
@@ -1202,7 +1218,7 @@ index_audit_header(struct group_index *index, bool fix)
         parent = &index->header->hash[bucket].recno;
         index_audit_loc(index, parent, bucket, NULL, fix);
         current = *parent;
-        while (current != -1) {
+        while (current >= 0 && current < index->count) {
             entry = &index->entries[current];
             next = &entry->next.recno;
             if (entry->deleted == 0 && bucket != index_bucket(entry->hash)) {
@@ -1230,6 +1246,8 @@ index_audit_header(struct group_index *index, bool fix)
                     reachable[current] = false;
                 }
             }
+            if (*next == current)
+                break;
             parent = next;
             current = *parent;
         }
@@ -1240,7 +1258,7 @@ index_audit_header(struct group_index *index, bool fix)
     index_audit_loc(index, &index->header->freelist.recno, 0, NULL, fix);
     parent = &index->header->freelist.recno;
     current = *parent;
-    while (current != -1) {
+    while (current >= 0 && current < index->count) {
         entry = &index->entries[current];
         index_audit_deleted(entry, current, fix);
         reachable[current] = true;
@@ -1252,6 +1270,8 @@ index_audit_header(struct group_index *index, bool fix)
             }
         }
         index_audit_loc(index, &entry->next.recno, current, entry, fix);
+        if (entry->next.recno == current)
+            break;
         parent = &entry->next.recno;
         current = *parent;
     }
@@ -1265,8 +1285,11 @@ index_audit_header(struct group_index *index, bool fix)
                 entry = &index->entries[current];
                 if (!HashEmpty(entry->hash) && entry->deleted == 0)
                     index_add(index, entry);
-                else
+                else {
+                    HashClear(&entry->hash);
+                    entry->deleted = 0;
                     freelist_add(index, entry);
+                }
             }
         }
 
@@ -1302,12 +1325,40 @@ index_audit_group(struct group_index *index, struct group_entry *entry,
             freelist_add(index, entry);
         }
     } else {
+        if (entry->flag != group->flag) {
+            entry->flag = group->flag;
+            mapcntl(entry, sizeof(*entry), MS_ASYNC);
+        }
         tdx_data_audit(group->name, entry, fix);
     }
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
 }
 
 
+/*
+**  Check to be sure that a given group exists in the overview index, and if
+**  missing, adds it.  Assumes that the index isn't locked, since it calls the
+**  normal functions for adding new groups (this should only be called after
+**  the index has already been repaired, for the same reason).  Called as a
+**  hash traversal function, walking the hash table of groups from the active
+**  file.
+*/
+static void
+index_audit_active(void *value, void *cookie)
+{
+    struct hashmap *group = value;
+    struct audit_data *data = cookie;
+    struct group_entry *entry;
+
+    entry = tdx_index_entry(data->index, group->name);
+    if (entry == NULL) {
+        warn("tradindexed: group %s missing from overview", group->name);
+        if (data->fix)
+            tdx_index_add(data->index, group->name, 0, 0, &group->flag);
+    }
+}
+
+
 /*
 **  Audit the group index for any inconsistencies.  If the argument is true,
 **  also attempt to fix those inconsistencies.
@@ -1322,6 +1373,7 @@ tdx_index_audit(bool fix)
     struct hash *hashmap;
     long bucket;
     struct group_entry *entry;
+    struct audit_data data;
 
     index = tdx_index_open(true);
     if (index == NULL)
@@ -1352,18 +1404,21 @@ tdx_index_audit(bool fix)
 
     /* Okay everything is now mapped and happy.  Validate the header. */
     index_audit_header(index, fix);
+    index_lock(index->fd, INN_LOCK_UNLOCK);
 
     /* Walk all the group entries and check them individually.  To do this, we
        need to map hashes to group names, so load a hash of the active file to
        do that resolution. */
     hashmap = hashmap_load();
+    data.index = index;
+    data.fix = fix;
+    hash_traverse(hashmap, index_audit_active, &data);
     for (bucket = 0; bucket < index->count; bucket++) {
         entry = &index->entries[bucket];
         if (HashEmpty(entry->hash) || entry->deleted != 0)
             continue;
         index_audit_group(index, entry, hashmap, fix);
     }
-    index_lock(index->fd, INN_LOCK_UNLOCK);
     if (hashmap != NULL)
         hash_free(hashmap);
 }
